@@ -9,18 +9,28 @@ from zope.interface import implements
 from twisted.internet import reactor, protocol, defer, error
 from twisted.python import log, util, reflect
 from twisted.protocols import amp
+from twisted.python import runtime
 
 from ampoule import iampoule
 
 gen = itertools.count()
 
+if runtime.platform.isWindows():
+    IS_WINDOWS = True
+    TO_CHILD = 0
+    FROM_CHILD = 1
+else:
+    IS_WINDOWS = False
+    TO_CHILD = 3
+    FROM_CHILD = 4
+
 class AMPConnector(protocol.ProcessProtocol):
     """
     A L{ProcessProtocol} subclass that can understand and speak AMP.
 
     @ivar amp: the children AMP process
     @type amp: L{amp.AMP}
-    
+
     @ivar finished: a deferred triggered when the process dies.
     @type finished: L{defer.Deferred}
 
@@ -32,7 +42,7 @@ class AMPConnector(protocol.ProcessProtocol):
         """
         @param proto: An instance or subclass of L{amp.AMP}
         @type proto: L{amp.AMP}
-        
+
         @param name: optional name of the subprocess.
         @type name: int
         """
@@ -45,7 +55,7 @@ class AMPConnector(protocol.ProcessProtocol):
     def signalProcess(self, signalID):
         """
         Send the signal signalID to the child process
-        
+
         @param signalID: The signal ID that you want to send to the
                         corresponding child
         @type signalID: C{str} or C{int}
@@ -55,16 +65,19 @@ class AMPConnector(protocol.ProcessProtocol):
     def connectionMade(self):
         log.msg("Subprocess %s started." % (self.name,))
         self.amp.makeConnection(self)
-        
+
     # Transport
     disconnecting = False
 
     def write(self, data):
-        self.transport.writeToChild(3, data)
+        if IS_WINDOWS:
+            self.transport.write(data)
+        else:
+            self.transport.writeToChild(TO_CHILD, data)
 
     def loseConnection(self):
-        self.transport.closeChildFD(3)
-        self.transport.closeChildfd(4)
+        self.transport.closeChildFD(TO_CHILD)
+        self.transport.closeChildFD(FROM_CHILD)
         self.transport.loseConnection()
 
     def getPeer(self):
@@ -74,7 +87,7 @@ class AMPConnector(protocol.ProcessProtocol):
         return ('no host',)
 
     def childDataReceived(self, childFD, data):
-        if childFD == 4:
+        if childFD == FROM_CHILD:
             self.amp.dataReceived(data)
             return
         self.errReceived(data)
@@ -97,23 +110,41 @@ import sys
 def main(reactor, ampChildPath):
     from twisted.application import reactors
     reactors.installReactor(reactor)
-    
+
     from twisted.python import log
     log.startLogging(sys.stderr)
 
     from twisted.internet import reactor, stdio
-    from twisted.python import reflect
+    from twisted.python import reflect, runtime
 
     ampChild = reflect.namedAny(ampChildPath)
-    stdio.StandardIO(ampChild(), 3, 4)
-    reactor.run()
+    if runtime.platform.isWindows():
+        stdio.StandardIO(ampChild(*sys.argv[1:-2]))
+    else:
+        stdio.StandardIO(ampChild(*sys.argv[1:-2]), %s, %s)
+    enter = getattr(ampChild, '__enter__', None)
+    if enter is not None:
+        enter()
+    try:
+        reactor.run()
+    except:
+        if enter is not None:
+            info = sys.exc_info()
+            if not ampChild.__exit__(*info):
+                raise
+        else:
+            raise
+    else:
+        if enter is not None:
+            ampChild.__exit__(None, None, None)
+
 main(sys.argv[-2], sys.argv[-1])
-"""
+""" % (TO_CHILD, FROM_CHILD)
 
 class ProcessStarter(object):
-    
+
     implements(iampoule.IStarter)
-    
+
     connectorFactory = AMPConnector
     def __init__(self, bootstrap=BOOTSTRAP, args=(), env={},
                  path=None, uid=None, gid=None, usePTY=0,
@@ -121,31 +152,31 @@ class ProcessStarter(object):
         """
         @param bootstrap: Startup code for the child process
         @type  bootstrap: C{str}
-        
+
         @param args: Arguments that should be supplied to every child
                      created.
         @type args: C{tuple} of C{str}
-        
+
         @param env: Environment variables that should be present in the
                     child environment
         @type env: C{dict}
-        
+
         @param path: Path in which to run the child
         @type path: C{str}
-        
+
         @param uid: if defined, the uid used to run the new process.
         @type uid: C{int}
-        
+
         @param gid: if defined, the gid used to run the new process.
         @type gid: C{int}
-        
+
         @param usePTY: Should the child processes use PTY processes
         @type usePTY: 0 or 1
-        
+
         @param packages: A tuple of packages that should be guaranteed
                          to be importable in the child processes
         @type packages: C{tuple} of C{str}
-        
+
         @param childReactor: a string that sets the reactor for child
                              processes
         @type childReactor: C{str}
@@ -194,11 +225,11 @@ class ProcessStarter(object):
             raise RuntimeError("importing %r is not the same as %r" %
                                (reflect.qual(obj), obj))
 
-    def startAMPProcess(self, ampChild, ampParent=None):
+    def startAMPProcess(self, ampChild, ampParent=None, ampChildArgs=()):
         """
         @param ampChild: a L{ampoule.child.AMPChild} subclass.
         @type ampChild: L{ampoule.child.AMPChild}
-    
+
         @param ampParent: an L{amp.AMP} subclass that implements the parent
                           protocol for this process pool
         @type ampParent: L{amp.AMP}
@@ -208,25 +239,25 @@ class ProcessStarter(object):
         if ampParent is None:
             ampParent = amp.AMP
         prot = self.connectorFactory(ampParent())
-        
-        return self.startPythonProcess(prot, self.childReactor, fullPath)
+        args = ampChildArgs + (self.childReactor, fullPath)
+        return self.startPythonProcess(prot, *args)
 
 
     def startPythonProcess(self, prot, *args):
         """
         @param prot: a L{protocol.ProcessProtocol} subclass
         @type prot: L{protocol.ProcessProtocol}
-    
+
         @param args: a tuple of arguments that will be added after the
                      ones in L{self.args} to start the child process.
-    
+
         @return: a tuple of the child process and the deferred finished.
                  finished triggers when the subprocess dies for any reason.
         """
         spawnProcess(prot, self.bootstrap, self.args+args, env=self.env,
                      path=self.path, uid=self.uid, gid=self.gid,
                      usePTY=self.usePTY, packages=self.packages)
-    
+
         # XXX: we could wait for startup here, but ... is there really any
         # reason to?  the pipe should be ready for writing.  The subprocess
         # might not start up properly, but then, a subprocess might shut down
@@ -255,6 +286,10 @@ def spawnProcess(processProtocol, bootstrap, args=(), env={},
     # that I'm using here, 3 and 4, so we are going to fix all these
     # issues when I add support for the configuration object that can
     # fix this stuff in a more configurable way.
-    return reactor.spawnProcess(processProtocol, sys.executable, args,
-                                env, path, uid, gid, usePTY,
-                                childFDs={0:"w", 1:"r", 2:"r", 3:"w", 4:"r"})
+    if IS_WINDOWS:
+        return reactor.spawnProcess(processProtocol, sys.executable, args,
+                                    env, path, uid, gid, usePTY)
+    else:
+        return reactor.spawnProcess(processProtocol, sys.executable, args,
+                                    env, path, uid, gid, usePTY,
+                                    childFDs={0:"w", 1:"r", 2:"r", 3:"w", 4:"r"})
