@@ -57,8 +57,9 @@ class ProcessPool(object):
 
     def __init__(self, ampChild=None, ampParent=None, min=5, max=20,
                  name=None, maxIdle=20, recycleAfter=500, starter=None,
-                 timeout=None):
+                 timeout=None, timeout_signal=DIE, ampChildArgs=()):
         self.starter = starter
+        self.ampChildArgs = tuple(ampChildArgs)
         if starter is None:
             self.starter = main.ProcessStarter(packages=("twisted", "ampoule"))
         self.ampParent = ampParent
@@ -72,6 +73,7 @@ class ProcessPool(object):
         self.maxIdle = maxIdle
         self.recycleAfter = recycleAfter
         self.timeout = timeout
+        self.timeout_signal = timeout_signal
         self._queue = []
         
         self.processes = set()
@@ -164,7 +166,7 @@ class ProcessPool(object):
         @type child: L{child.AMPChild}
         """
         try:
-            child.transport.signalProcess(DIE)
+            child.transport.signalProcess(self.timeout_signal)
         except error.ProcessExitedAlready:
             # don't do anything then... we are too late
             # or we were too early to call
@@ -188,11 +190,14 @@ class ProcessPool(object):
             # You might end up with a dirty reactor due to the stop()
             # returning before the new process is created.
             return
-        child, finished = self.starter.startAMPProcess(self.ampChild,
-                                                       ampParent=self.ampParent)
+        startAMPProcess = self.starter.startAMPProcess
+        child, finished = startAMPProcess(self.ampChild,
+                                          ampParent=self.ampParent,
+                                          ampChildArgs=self.ampChildArgs)
         return self._addProcess(child, finished)
     
-    def _cb_doWork(self, command, _d=None, _timeout=None, **kwargs):
+    def _cb_doWork(self, command, _d=None, _timeout=None, _deadline=None,
+                   **kwargs):
         """
         Go and call the command.
         
@@ -204,12 +209,18 @@ class ProcessPool(object):
         
         @param _timeout: The timeout for this call only
         @type _timeout: C{int}
+        @param _deadline: The deadline for this call only
+        @type _deadline: C{int}
         """
         timeoutCall = None
+        deadlineCall = None
         
         def _returned(result, child, is_error=False):
-            if timeoutCall is not None and timeoutCall.active():
-                timeoutCall.cancel()
+            def cancelCall(call):
+                if call is not None and call.active():
+                    call.cancel()
+            cancelCall(timeoutCall)
+            cancelCall(deadlineCall)
             self.busy.discard(child)
             if not die:
                 # we are not marked to be removed, so add us back to
@@ -250,7 +261,7 @@ class ProcessPool(object):
         # We also need to guard against timeout errors for child
         # and local timeout parameter overrides the global one
         if _timeout == 0:
-            timeout = timeout
+            timeout = _timeout
         else:
             timeout = _timeout or self.timeout
 
@@ -258,6 +269,12 @@ class ProcessPool(object):
             from twisted.internet import reactor
             timeoutCall = reactor.callLater(timeout, self._handleTimeout, child)
 
+        if _deadline is not None:
+            from twisted.internet import reactor
+            delay = max(0, _deadline - reactor.seconds())
+            deadlineCall = reactor.callLater(delay, self._handleTimeout,
+                                             child)
+
         return defer.maybeDeferred(child.callRemote, command, **kwargs
             ).addCallback(_returned, child
             ).addErrback(_returned, child, is_error=True)
@@ -333,7 +350,6 @@ class ProcessPool(object):
         if len(self.processes) < self.max:
             self.startAWorker()
 
-    @defer.inlineCallbacks
     def adjustPoolSize(self, min=None, max=None):
         """
         Change the pool size to be at least min and less than max,
@@ -351,24 +367,27 @@ class ProcessPool(object):
         self.min = min
         self.max = max
         
+        l = []
         if self.started:
-            while len(self.processes) > self.max:
-                yield self.stopAWorker()
+            
+            for i in xrange(len(self.processes)-self.max):
+                l.append(self.stopAWorker())
             while len(self.processes) < self.min:
                 self.startAWorker()
-        self.dumpStats()
+        
+        return defer.DeferredList(l).addCallback(lambda _: self.dumpStats())
 
-    @defer.inlineCallbacks
     def stop(self):
         """
         Stops the process protocol.
         """
         self.finished = True
         l = [self.stopAWorker(process) for process in self.processes]
-        d = defer.DeferredList(l)
-        yield d
-        if self.looping.running:
-            self.looping.stop()
+        def _cb(_):
+            if self.looping.running:
+                self.looping.stop()
+
+        return defer.DeferredList(l).addCallback(_cb)
 
     def dumpStats(self):
         log.msg("ProcessPool stats:")
