@@ -1,5 +1,10 @@
+
+from signal import SIGHUP
 import math
+import os
+import os.path
 from cStringIO import StringIO as sio
+import tempfile
 
 from twisted.internet import error, defer, reactor
 from twisted.python import failure, reflect
@@ -120,6 +125,46 @@ class BadChild(child.AMPChild):
         return {}
     Die.responder(die)
 
+
+class Write(amp.Command):
+    response = [("response", amp.String())]
+    pass
+
+
+class Writer(child.AMPChild):
+
+    def __init__(self, data='hello'):
+        child.AMPChild.__init__(self)
+        self.data = data
+
+    def write(self):
+        return {'response': self.data}
+    Write.responder(write)
+
+
+class GetCWD(amp.Command):
+
+    response = [("cwd", amp.String())]
+
+
+class TempDirChild(child.AMPChild):
+
+    @staticmethod
+    def __enter__():
+        directory = tempfile.mkdtemp()
+        os.chdir(directory)
+
+    @staticmethod
+    def __exit__(exc_type, exc_val, exc_tb):
+        cwd = os.getcwd()
+        os.chdir('..')
+        os.rmdir(cwd)
+
+    def getcwd(self):
+        return {'cwd': os.getcwd()}
+    GetCWD.responder(getcwd)
+
+
 class TestAMPConnector(unittest.TestCase):
     def setUp(self):
         """
@@ -252,6 +297,19 @@ main()
            ).addCallback(lambda _: c.callRemote(commands.Shutdown))
         return finished
 
+    def test_BootstrapContext(self):
+        starter = main.ProcessStarter(packages=('twisted', 'ampoule'))
+        c, finished = starter.startAMPProcess(TempDirChild)
+        cwd = []
+        def checkBootstrap(response):
+            cwd.append(response['cwd'])
+            self.assertNotEquals(cwd, os.getcwd())
+        c.callRemote(GetCWD
+            ).addCallback(checkBootstrap
+            ).addCallback(lambda _: c.callRemote(commands.Shutdown)
+            ).addCallback(lambda _: self.assertFalse(os.path.exists(cwd[0])))
+        return finished
+
     def test_startAMPAndParentProtocol(self):
         """
         Test that you can start an AMP subprocess and the children can
@@ -366,12 +424,9 @@ class TestProcessPool(unittest.TestCase):
             self.assertEquals(len(pp._finishCallbacks), pp.max)
         
         def _resize3(_):
-            return self.assertFailure(pp.adjustPoolSize(min=-1, max=5), AssertionError
-                ).addCallback(lambda _:
-                    self.assertFailure(pp.adjustPoolSize(min=5, max=1), AssertionError)
-                ).addCallback(lambda _:
-                    pp.stop()
-                )
+            self.assertRaises(AssertionError, pp.adjustPoolSize, min=-1, max=5)
+            self.assertRaises(AssertionError, pp.adjustPoolSize, min=5, max=1)
+            return pp.stop()
         
         return pp.start(
             ).addCallback(_resize1
@@ -674,16 +729,81 @@ class TestProcessPool(unittest.TestCase):
             ).addCallback(_check
             ).addCallback(lambda _: pp.stop())
 
+    def test_SupplyChildArgs(self):
+        """Ensure that arguments for the child constructor are passed in."""
+        pp = pool.ProcessPool(Writer, ampChildArgs=['body'], min=0)
+        def _check(result):
+            return pp.doWork(Write).addCallback(
+            self.assertEquals, {'response': 'body'})
+
+        return pp.start(
+            ).addCallback(_check
+            ).addCallback(lambda _: pp.stop())
+
+    def processTimeoutTest(self, timeout):
+        pp = pool.ProcessPool(WaitingChild, min=1, max=1)
+        
+        def _work(_):
+            d = pp.callRemote(First, data="ciao", _timeout=timeout)
+            self.assertFailure(d, error.ProcessTerminated)
+            return d
+
+        return pp.start(
+            ).addCallback(_work
+            ).addCallback(lambda _: pp.stop())
+
     def test_processTimeout(self):
         """
         Test that a call that doesn't finish within the given timeout
         time is correctly handled.
         """
+        return self.processTimeoutTest(1)
+
+    def test_processTimeoutZero(self):
+        """
+        Test that the process is correctly handled when the timeout is zero.
+        """
+        return self.processTimeoutTest(0)
+
+    def test_processDeadline(self):
         pp = pool.ProcessPool(WaitingChild, min=1, max=1)
+
+        def _work(_):
+            d = pp.callRemote(First, data="ciao", _deadline=reactor.seconds())
+            self.assertFailure(d, error.ProcessTerminated)
+            return d
+
+        return pp.start(
+            ).addCallback(_work
+            ).addCallback(lambda _: pp.stop())
+
+    def test_processBeforeDeadline(self):
+        pp = pool.ProcessPool(PidChild, min=1, max=1)
+
+        def _work(_):
+            d = pp.callRemote(Pid, _deadline=reactor.seconds() + 10)
+            d.addCallback(lambda result: self.assertNotEqual(result['pid'], 0))
+            return d
+
+        return pp.start(
+            ).addCallback(_work
+            ).addCallback(lambda _: pp.stop())
+
+    def test_processTimeoutSignal(self):
+        """
+        Test that a call that doesn't finish within the given timeout
+        time is correctly handled.
+        """
+        pp = pool.ProcessPool(WaitingChild, min=1, max=1,
+                              timeout_signal=SIGHUP)
         
         def _work(_):
             d = pp.callRemote(First, data="ciao", _timeout=1)
-            self.assertFailure(d, error.ProcessTerminated)
+            d.addCallback(lambda d: self.fail())
+            text = 'signal %d' % SIGHUP
+            d.addErrback(
+                lambda f: self.assertTrue(text in f.value[0],
+                '"%s" not in "%s"' % (text, f.value[0])))
             return d
 
         return pp.start(
